//===============================================================
//    
//===============================================================

#include "make.h"
#include "xml_lib.h"


//-----------------------------------------------------------------------------
//     
FILE* output = stdout;


//-----------------------------------------------------------------------------
//     ( /)
class CResourceManager {
	//       
	// output_file != NULL,      output
	FILE* output_file;
public:
	//      stdout
	bool SetOutput(const char* fname);
	// ( )
	CResourceManager() : output_file(NULL)
	{};
	// ( )
	~CResourceManager()
	{
		//      (.  '-f')
		if (output_file) {
			output = stdout;
			if (fclose(output_file)) fprintf(output, "FILE I/O: Cannot close file, specified by switch -f\n");
		}
	};
} ResourceManager;


//-----------------------------------------------------------------------------
//      stdout
bool CResourceManager::SetOutput(const char *fname)
{
	//     
	output_file = fopen(fname, "w");
	if (!output_file) {
		fprintf(output, "FILE I/O: Cannot open file %s for writing\n", fname);
		return false;
	}
	//     
	output = output_file;
	return true;
}


//---------------------------------------------------------------
//   
int main(int argc, char *argv[])
{
	// Make2M
	const char* const Version = "1.8.13";

	// ,    
	const int ErrExitCode = 1;

	//   ,    
	bool sw_about = true;
	bool sw_version = false;
	bool sw_help = false;
	bool sw_arg_err = false;
	bool sw_win_to_oem = false;
	//  -   Win32 Console Application (MS VC++ 6.0)
	int target = 0;
	//        C++
	const char* dir_name = "../Make";
	//    c++
	const char* cpp_path = "c++";
	//      c++
	const char* cpp_options = "-fconserve-space -O3 -s";

	//   
	if (argc > 1) {
		for (int i = 1; i < argc; i++)
			if ('-' == argv[i][0] || '/' == argv[i][0]) {

				//  'p'
				if ('P' == toupper(argv[i][1]) && '\0' == argv[i][2]) {
					//      
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found target platform after -p in command line\n");
						sw_arg_err = true;
						break;
					}
					switch (argv[i][0]) {
					case '0': break;
					case '1': target = 1; break;
					case '2': target = 2; break;
					case '3': target = 3; break;
					default:
						fprintf(output, "ERROR: Unknown target platform\n");
						sw_arg_err = true;
						break;
					}
				} else

				//  'c', 'cp'  'ca'
				if ('C' == toupper(argv[i][1])) {
					//   'c'  'cp'
					if (argv[i][2] == '\0')
						sw_win_to_oem = true;
					else
						if ('P' == toupper(argv[i][2]) && '\0' == argv[i][3]) {
							//    c++    
							if (++i >= argc) {
								fprintf(output, "ERROR: Not found file name after -cp in command line\n");
								sw_arg_err = true;
								break;
							}
							cpp_path = argv[i];
						} else
						if ('A' == toupper(argv[i][2]) && '\0' == argv[i][3]) {
							//   c++    
							if (++i >= argc) {
								fprintf(output, "ERROR: Not found argument after -ca in command line\n");
								sw_arg_err = true;
								break;
							}
							cpp_options = argv[i];
						} else {
							fprintf(output, "ERROR: Unknown command line option %s\n", argv[i]);
							sw_arg_err = true;
							break;
						}
				} else

				//  'o'
				if ('O' == toupper(argv[i][1]) && '\0' == argv[i][2]) {
					//     
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found path after -o in command line\n");
						sw_arg_err = true;
						break;
					}
					dir_name = argv[i];
				} else

				//  'a'
				if ('A' == toupper(argv[i][1]) && '\0' == argv[i][2]) sw_about = false;
				else

				//  'f'
				if ('F' == toupper(argv[i][1]) && '\0' == argv[i][2]) {
					//      
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found file name after -f in command line\n");
						sw_arg_err = true;
						break;
					}
					//        
					if (!ResourceManager.SetOutput(argv[i])) return ErrExitCode;
				} else

				//  'v'
				if ('V' == toupper(argv[i][1]) && '\0' == argv[i][2]) sw_version = true;
				else

				//  'h'
				if (('H' == toupper(argv[i][1]) || '?' == argv[i][1]) && '\0' == argv[i][2])
					sw_help = true;
				else {
					fprintf(output, "ERROR: Unknown command line option %s\n", argv[i]);
					sw_arg_err = true;
					break;
				}
			}
	} else sw_help = true;

	//   ( )
	if (sw_version) {
		fprintf(output, "Make2M version %s\n", Version);
		return 0;
	}
	//    -  
	if (!sw_help && !sw_arg_err && (argc < 3 || '-' == argv[1][0] || '/' == argv[1][0] || '-' == argv[2][0] || '/' == argv[2][0])) {
		fprintf(output, "ERROR: Invalid command line\n");
		sw_arg_err = true;
	}
	//   ( )
	if (sw_about) fprintf(output, "\nMake2M (O2M makefile creator) v%s\nCopyright(c) Schwetz Dmitry, 2005\n\n", Version);
	if (sw_help || sw_arg_err) {
		fprintf(output, "Usage:      Make2M <O2M_makefile> <makefile_name> [-<switch>...]\n\n<Examples>\n  Make2M _O2M_make.2mk Makefile.mak\n  Make2M _O2M_make.2mk \"Make file.mak\" -f \"log.txt\" -a\n\n<Switches>\n");
		fprintf(output, "  p <digit> Set target platform:\n");
		fprintf(output, "  p 0       Win32 Console Application (MS VC++ 6.0) (default)\n");
		fprintf(output, "  p 1       Win32 Application (MS VC++ 6.0)\n");
		fprintf(output, "  p 2       Console Application (MinGW 3.1.0)\n");
		fprintf(output, "  p 3       Win32 Application (MinGW 3.1.0)\n");
		fprintf(output, "  c         File name conversion from Win1251 to OEM Cyrillic (only with -p 2,3)\n");
		fprintf(output, "  cp <file> Specify path for c++ compiler (only with -p 2,3, \"c++\" by default)\n");
		fprintf(output, "  ca <arg>  Pass <arg> to c++ compiler (only with -p 2,3)\n");
		fprintf(output, "  o <path>  Set output directory (\"../Make\" by default)\n");
		fprintf(output, "  a         Disable about info\n");
		fprintf(output, "  f <file>  Specify file for output messages instead of stdout\n");
		fprintf(output, "  v         Version info\n");
		fprintf(output, "  h,?       This help\n");
		return sw_arg_err ? ErrExitCode : 0;
	}


	////////////////////////
	// O2M makefile

	//    makefile
	FILE* f;
	//      
	const char* m_name;
	//     
	const char* exec_name;

	// XML      ( _O2M_make.2mk)
	CXMLFile xmlFile;
	if (argc > 1 && '-' != argv[1][0] && '/' != argv[1][0])
		switch (xmlFile.Load(argv[1])) {
		case EXMLNoError:
			break;
		case EXMLFormat:
			fprintf(output, "ERROR: Incorrect O2M makefile (\"%s\") file format\n", argv[1]);
			goto fault_exit;
		case EXMLOpen:
			fprintf(output, "ERROR: Cannot open file \"%s\" for reading\n", argv[1]);
			goto fault_exit;
		case EXMLClose:
			fprintf(output, "ERROR: Cannot close file \"%s\"\n", argv[1]);
			goto fault_exit;
		default:
			fprintf(output, "ERROR: UNKNOWN INTERNAL ERROR\n");
			goto fault_exit;
		}
	else {
		fprintf(output, "ERROR: Not specified O2M makefile\n");
		goto fault_exit;
	}

	//    
	m_name = xmlFile.XMLEntry->AttribValue("Main");
	if (!m_name) {
		fprintf(output, "ERROR: Not found name of main module in O2M makefile (\"%s\")\n", argv[1]);
		goto fault_exit;
	}

	//   
	exec_name = xmlFile.XMLEntry->AttribValue("Exec");
	if (!exec_name) exec_name = m_name;

	//     makefile
	f = fopen(argv[2], "w");
	if (!f) {
		fprintf(output, "ERROR: Cannot open file \"%s\" for writing\n", argv[2]);
		goto fault_exit;
	}

	//    
	bool res;

	//  
	switch (target) {
	case 0:
		// Makefile     Windows
		res = make_Win32_Console_makefile(f, xmlFile.XMLEntry, dir_name, exec_name, m_name);
		break;
	case 1:
		// Makefile     Windows
		res = make_Win32_makefile(f, xmlFile.XMLEntry, dir_name, exec_name, m_name);
		break;
	case 2:
		// Makefile    MinGW
		res = make_MinGW_makefile(f, xmlFile.XMLEntry, dir_name, exec_name, m_name, cpp_path, cpp_options, sw_win_to_oem, false);
		break;
	case 3:
		// Makefile    MinGW
		res = make_MinGW_makefile(f, xmlFile.XMLEntry, dir_name, exec_name, m_name, cpp_path, cpp_options, sw_win_to_oem, true);
		break;
	}//switch

	//  makefile
	if (fclose(f)) {
		fprintf(output, "ERROR: Cannot close file \"%s\"\n", argv[2]);
		goto fault_exit;
	};

	//     .mak 
	if (res) {
		fprintf(output, "Make2M - OK\n");
		return 0;
	}

fault_exit:
	fprintf(output, "Make2M - FAILED\n");
	return ErrExitCode;
}
